@@ -1,12 +1,13 @@
-extern crate clap;
+use clap;
 
 use clap::{AppSettings, Arg, ArgMatches};
 
+use cargo::core::features;
 use cargo::{self, CliResult, Config};
 
 use super::commands;
 use super::list_commands;
-use command_prelude::*;
+use crate::command_prelude::*;
 
 pub fn main(config: &mut Config) -> CliResult {
     let args = match cli().get_matches_safe() {
@@ -31,12 +32,25 @@ Available unstable (nightly-only) flags:
     -Z avoid-dev-deps   -- Avoid installing dev-dependencies if possible
     -Z minimal-versions -- Install minimal dependency versions instead of maximum
     -Z no-index-update  -- Do not update the registry, avoids a network request for benchmarking
-    -Z offline          -- Offline mode that does not perform network requests
     -Z unstable-options -- Allow the usage of unstable options such as --registry
     -Z config-profile   -- Read profiles from .cargo/config files
+    -Z install-upgrade  -- `cargo install` will upgrade instead of failing
 
 Run with 'cargo -Z [FLAG] [SUBCOMMAND]'"
         );
+        if !features::nightly_features_allowed() {
+            println!(
+                "\nUnstable flags are only available on the nightly channel \
+                 of Cargo, but this is the `{}` channel.\n\
+                 {}",
+                features::channel(),
+                features::SEE_CHANNELS
+            );
+        }
+        println!(
+            "\nSee https://doc.rust-lang.org/nightly/cargo/reference/unstable.html \
+             for more information about these flags."
+        );
         return Ok(());
     }
 
@@ -47,8 +61,8 @@ Run with 'cargo -Z [FLAG] [SUBCOMMAND]'"
         return Ok(());
     }
 
-    if let Some(ref code) = args.value_of("explain") {
-        let mut procss = config.rustc(None)?.process();
+    if let Some(code) = args.value_of("explain") {
+        let mut procss = config.load_global_rustc(None)?.process();
         procss.arg("--explain").arg(code).exec()?;
         return Ok(());
     }
@@ -132,7 +146,7 @@ fn expand_aliases(
     Ok(args)
 }
 
-fn execute_subcommand(config: &mut Config, args: &ArgMatches) -> CliResult {
+fn execute_subcommand(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
     let (cmd, subcommand_args) = match args.subcommand() {
         (cmd, Some(args)) => (cmd, args),
         _ => {
@@ -145,7 +159,7 @@ fn execute_subcommand(config: &mut Config, args: &ArgMatches) -> CliResult {
 
     config.configure(
         args.occurrences_of("verbose") as u32,
-        if args.is_present("quiet") {
+        if args.is_present("quiet") || subcommand_args.is_present("quiet") {
             Some(true)
         } else {
             None
@@ -153,6 +167,7 @@ fn execute_subcommand(config: &mut Config, args: &ArgMatches) -> CliResult {
         &args.value_of("color").map(|s| s.to_string()),
         args.is_present("frozen"),
         args.is_present("locked"),
+        args.is_present("offline"),
         arg_target_dir,
         &args
             .values_of_lossy("unstable-features")
@@ -176,7 +191,6 @@ fn cli() -> App {
             AppSettings::VersionlessSubcommands,
             AppSettings::AllowExternalSubcommands,
         ])
-        .about("")
         .template(
             "\
 Rust's package manager
@@ -194,7 +208,7 @@ Some common cargo commands are (see all commands with --list):
     doc         Build this package's and its dependencies' documentation
     new         Create a new cargo package
     init        Create a new cargo package in an existing directory
-    run         Build and execute src/main.rs
+    run         Run a binary or example of the local package
     test        Run the tests
     bench       Run the benchmarks
     update      Update dependencies listed in Cargo.lock
@@ -217,11 +231,7 @@ See 'cargo help <command>' for more information on a specific command.\n",
             .multiple(true)
             .global(true),
         )
-        .arg(
-            opt("quiet", "No output printed to stdout")
-                .short("q")
-                .global(true),
-        )
+        .arg(opt("quiet", "No output printed to stdout").short("q"))
         .arg(
             opt("color", "Coloring: auto, always, never")
                 .value_name("WHEN")
@@ -229,6 +239,7 @@ See 'cargo help <command>' for more information on a specific command.\n",
         )
         .arg(opt("frozen", "Require Cargo.lock and cache are up to date").global(true))
         .arg(opt("locked", "Require Cargo.lock is up to date").global(true))
+        .arg(opt("offline", "Run without accessing the network").global(true))
         .arg(
             Arg::with_name("unstable-features")
                 .help("Unstable (nightly-only) flags to Cargo, see 'cargo -Z help' for details")
