@@ -6,7 +6,7 @@ use std::env;
 use std::fmt;
 use std::fs::{self, File};
 use std::io::prelude::*;
-use std::io::SeekFrom;
+use std::io::{self, SeekFrom};
 use std::mem;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
@@ -25,12 +25,12 @@ use crate::core::profiles::ConfigProfiles;
 use crate::core::shell::Verbosity;
 use crate::core::{CliUnstable, Shell, SourceId, Workspace};
 use crate::ops;
-use crate::util::errors::{internal, CargoResult, CargoResultExt};
+use crate::util::errors::{self, internal, CargoResult, CargoResultExt};
 use crate::util::toml as cargo_toml;
 use crate::util::Filesystem;
 use crate::util::Rustc;
-use crate::util::ToUrl;
-use crate::util::{paths, validate_package_name};
+use crate::util::{paths, validate_package_name, FileLock};
+use crate::util::{ToUrl, ToUrlWithBase};
 
 /// Configuration information for cargo. This is not specific to a build, it is information
 /// relating to cargo itself.
@@ -52,10 +52,15 @@ pub struct Config {
     rustdoc: LazyCell<PathBuf>,
     /// Whether we are printing extra verbose messages
     extra_verbose: bool,
-    /// `frozen` is set if we shouldn't access the network
+    /// `frozen` is the same as `locked`, but additionally will not access the
+    /// network to determine if the lock file is out-of-date.
     frozen: bool,
-    /// `locked` is set if we should not update lock files
+    /// `locked` is set if we should not update lock files. If the lock file
+    /// is missing, or needs to be updated, an error is produced.
     locked: bool,
+    /// `offline` is set if we should never access the network, but otherwise
+    /// continue operating if possible.
+    offline: bool,
     /// A global static IPC control mechanism (used for managing parallel builds)
     jobserver: Option<jobserver::Client>,
     /// Cli flags of the form "-Z something"
@@ -74,6 +79,11 @@ pub struct Config {
     env: HashMap<String, String>,
     /// Profiles loaded from config.
     profiles: LazyCell<ConfigProfiles>,
+    /// Tracks which sources have been updated to avoid multiple updates.
+    updated_sources: LazyCell<RefCell<HashSet<SourceId>>>,
+    /// Lock, if held, of the global package cache along with the number of
+    /// acquisitions so far.
+    package_cache_lock: RefCell<Option<(FileLock, usize)>>,
 }
 
 impl Config {
@@ -114,6 +124,7 @@ impl Config {
             extra_verbose: false,
             frozen: false,
             locked: false,
+            offline: false,
             jobserver: unsafe {
                 if GLOBAL_JOBSERVER.is_null() {
                     None
@@ -129,6 +140,8 @@ impl Config {
             target_dir: None,
             env,
             profiles: LazyCell::new(),
+            updated_sources: LazyCell::new(),
+            package_cache_lock: RefCell::new(None),
         }
     }
 
@@ -271,6 +284,12 @@ impl Config {
         })
     }
 
+    pub fn updated_sources(&self) -> RefMut<'_, HashSet<SourceId>> {
+        self.updated_sources
+            .borrow_with(|| RefCell::new(HashSet::new()))
+            .borrow_mut()
+    }
+
     pub fn values(&self) -> CargoResult<&HashMap<String, ConfigValue>> {
         self.values.try_borrow_with(|| self.load_values())
     }
@@ -547,6 +566,7 @@ impl Config {
         color: &Option<String>,
         frozen: bool,
         locked: bool,
+        offline: bool,
         target_dir: &Option<PathBuf>,
         unstable_flags: &[String],
     ) -> CargoResult<()> {
@@ -591,6 +611,11 @@ impl Config {
         self.extra_verbose = extra_verbose;
         self.frozen = frozen;
         self.locked = locked;
+        self.offline = offline
+            || self
+                .get::<Option<bool>>("net.offline")
+                .unwrap_or(None)
+                .unwrap_or(false);
         self.target_dir = cli_target_dir;
         self.cli_flags.parse(unstable_flags)?;
 
@@ -606,7 +631,11 @@ impl Config {
     }
 
     pub fn network_allowed(&self) -> bool {
-        !self.frozen() && !self.cli_unstable().offline
+        !self.frozen() && !self.offline()
+    }
+
+    pub fn offline(&self) -> bool {
+        self.offline
     }
 
     pub fn frozen(&self) -> bool {
@@ -658,18 +687,34 @@ impl Config {
         validate_package_name(registry, "registry name", "")?;
         Ok(
             match self.get_string(&format!("registries.{}.index", registry))? {
-                Some(index) => {
-                    let url = index.val.to_url()?;
-                    if url.password().is_some() {
-                        failure::bail!("Registry URLs may not contain passwords");
-                    }
-                    url
-                }
+                Some(index) => self.resolve_registry_index(index)?,
                 None => failure::bail!("No index found for registry: `{}`", registry),
             },
         )
     }
 
+    /// Gets the index for the default registry.
+    pub fn get_default_registry_index(&self) -> CargoResult<Option<Url>> {
+        Ok(match self.get_string("registry.index")? {
+            Some(index) => Some(self.resolve_registry_index(index)?),
+            None => None,
+        })
+    }
+
+    fn resolve_registry_index(&self, index: Value<String>) -> CargoResult<Url> {
+        let base = index
+            .definition
+            .root(&self)
+            .join("truncated-by-url_with_base");
+        // Parse val to check it is a URL, not a relative path without a protocol.
+        let _parsed = index.val.to_url()?;
+        let url = index.val.to_url_with_base(Some(&*base))?;
+        if url.password().is_some() {
+            failure::bail!("Registry URLs may not contain passwords");
+        }
+        Ok(url)
+    }
+
     /// Loads credentials config from the credentials file into the `ConfigValue` object, if
     /// present.
     fn load_credentials(&self, cfg: &mut ConfigValue) -> CargoResult<()> {
@@ -803,6 +848,86 @@ impl Config {
         };
         T::deserialize(d).map_err(|e| e.into())
     }
+
+    pub fn assert_package_cache_locked<'a>(&self, f: &'a Filesystem) -> &'a Path {
+        let ret = f.as_path_unlocked();
+        assert!(
+            self.package_cache_lock.borrow().is_some(),
+            "pacakge cache lock is not currently held, Cargo forgot to call \
+             `acquire_package_cache_lock` before we got to this stack frame",
+        );
+        assert!(ret.starts_with(self.home_path.as_path_unlocked()));
+        return ret;
+    }
+
+    /// Acquires an exclusive lock on the global "package cache"
+    ///
+    /// This lock is global per-process and can be acquired recursively. An RAII
+    /// structure is returned to release the lock, and if this process
+    /// abnormally terminates the lock is also released.
+    pub fn acquire_package_cache_lock<'a>(&'a self) -> CargoResult<PackageCacheLock<'a>> {
+        let mut slot = self.package_cache_lock.borrow_mut();
+        match *slot {
+            // We've already acquired the lock in this process, so simply bump
+            // the count and continue.
+            Some((_, ref mut cnt)) => {
+                *cnt += 1;
+            }
+            None => {
+                let path = ".package-cache";
+                let desc = "package cache lock";
+
+                // First, attempt to open an exclusive lock which is in general
+                // the purpose of this lock!
+                //
+                // If that fails because of a readonly filesystem, though, then
+                // we don't want to fail because it's a readonly filesystem. In
+                // some situations Cargo is prepared to have a readonly
+                // filesystem yet still work since it's all been pre-downloaded
+                // and/or pre-unpacked. In these situations we want to keep
+                // Cargo running if possible, so if it's a readonly filesystem
+                // switch to a shared lock which should hopefully succeed so we
+                // can continue.
+                //
+                // Note that the package cache lock protects files in the same
+                // directory, so if it's a readonly filesystem we assume that
+                // the entire package cache is readonly, so we're just acquiring
+                // something to prove it works, we're not actually doing any
+                // synchronization at that point.
+                match self.home_path.open_rw(path, self, desc) {
+                    Ok(lock) => *slot = Some((lock, 1)),
+                    Err(e) => {
+                        if maybe_readonly(&e) {
+                            if let Ok(lock) = self.home_path.open_ro(path, self, desc) {
+                                *slot = Some((lock, 1));
+                                return Ok(PackageCacheLock(self));
+                            }
+                        }
+
+                        Err(e).chain_err(|| "failed to acquire package cache lock")?;
+                    }
+                }
+            }
+        }
+        return Ok(PackageCacheLock(self));
+
+        fn maybe_readonly(err: &failure::Error) -> bool {
+            err.iter_chain().any(|err| {
+                if let Some(io) = err.downcast_ref::<io::Error>() {
+                    if io.kind() == io::ErrorKind::PermissionDenied {
+                        return true;
+                    }
+
+                    #[cfg(unix)]
+                    return io.raw_os_error() == Some(libc::EROFS);
+                }
+
+                false
+            })
+        }
+    }
+
+    pub fn release_package_cache_lock(&self) {}
 }
 
 /// A segment of a config key.
@@ -932,12 +1057,7 @@ impl std::error::Error for ConfigError {}
 // `cause` and avoid doing the cause formatting here.
 impl fmt::Display for ConfigError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let message = self
-            .error
-            .iter_chain()
-            .map(|e| e.to_string())
-            .collect::<Vec<_>>()
-            .join("\nCaused by:\n  ");
+        let message = errors::display_causes(&self.error);
         if let Some(ref definition) = self.definition {
             write!(f, "error in {}: {}", definition, message)
         } else {
@@ -1644,3 +1764,16 @@ pub fn save_credentials(cfg: &Config, token: String, registry: Option<String>) -
         Ok(())
     }
 }
+
+pub struct PackageCacheLock<'a>(&'a Config);
+
+impl Drop for PackageCacheLock<'_> {
+    fn drop(&mut self) {
+        let mut slot = self.0.package_cache_lock.borrow_mut();
+        let (_, cnt) = slot.as_mut().unwrap();
+        *cnt -= 1;
+        if *cnt == 0 {
+            *slot = None;
+        }
+    }
+}
