@@ -1,5 +1,5 @@
-use support::registry::Package;
-use support::{basic_bin_manifest, basic_manifest, main_file, project};
+use crate::support::registry::Package;
+use crate::support::{basic_bin_manifest, basic_manifest, main_file, project};
 
 #[test]
 fn cargo_build_plan_simple() {
@@ -34,7 +34,8 @@ fn cargo_build_plan_simple() {
         ]
     }
     "#,
-        ).run();
+        )
+        .run();
     assert!(!p.bin("foo").is_file());
 }
 
@@ -52,7 +53,8 @@ fn cargo_build_plan_single_dep() {
             [dependencies]
             bar = { path = "bar" }
         "#,
-        ).file(
+        )
+        .file(
             "src/lib.rs",
             r#"
             extern crate bar;
@@ -61,7 +63,8 @@ fn cargo_build_plan_single_dep() {
             #[test]
             fn test() { foo(); }
         "#,
-        ).file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
+        )
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
     p.cargo("build --build-plan -Zunstable-options")
@@ -82,7 +85,8 @@ fn cargo_build_plan_single_dep() {
                 "kind": "Host",
                 "links": "{...}",
                 "outputs": [
-                    "[..]/foo/target/debug/deps/libbar-[..].rlib"
+                    "[..]/foo/target/debug/deps/libbar-[..].rlib",
+                    "[..]/foo/target/debug/deps/libbar-[..].rmeta"
                 ],
                 "package_name": "bar",
                 "package_version": "0.0.1",
@@ -98,7 +102,8 @@ fn cargo_build_plan_single_dep() {
                 "kind": "Host",
                 "links": "{...}",
                 "outputs": [
-                    "[..]/foo/target/debug/deps/libfoo-[..].rlib"
+                    "[..]/foo/target/debug/deps/libfoo-[..].rlib",
+                    "[..]/foo/target/debug/deps/libfoo-[..].rmeta"
                 ],
                 "package_name": "foo",
                 "package_version": "0.5.0",
@@ -109,7 +114,8 @@ fn cargo_build_plan_single_dep() {
         ]
     }
     "#,
-        ).run();
+        )
+        .run();
 }
 
 #[test]
@@ -125,7 +131,8 @@ fn cargo_build_plan_build_script() {
             authors = ["wycats@example.com"]
             build = "build.rs"
         "#,
-        ).file("src/main.rs", r#"fn main() {}"#)
+        )
+        .file("src/main.rs", r#"fn main() {}"#)
         .file("build.rs", r#"fn main() {}"#)
         .build();
 
@@ -185,7 +192,8 @@ fn cargo_build_plan_build_script() {
         ]
     }
     "#,
-        ).run();
+        )
+        .run();
 }
 
 #[test]
