@@ -1,48 +1,42 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
 extern crate clap;
 extern crate clippy_dev;
 extern crate regex;
 
-use clap::{App, Arg, SubCommand};
+use clap::{App, AppSettings, Arg, SubCommand};
 use clippy_dev::*;
 
 #[derive(PartialEq)]
 enum UpdateMode {
     Check,
-    Change
+    Change,
 }
 
 fn main() {
     let matches = App::new("Clippy developer tooling")
+        .setting(AppSettings::SubcommandRequiredElseHelp)
         .subcommand(
             SubCommand::with_name("update_lints")
-                .about("Makes sure that:\n \
-                       * the lint count in README.md is correct\n \
-                       * the changelog contains markdown link references at the bottom\n \
-                       * all lint groups include the correct lints\n \
-                       * lint modules in `clippy_lints/*` are visible in `src/lib.rs` via `pub mod`\n \
-                       * all lints are registered in the lint store")
-                .arg(
-                    Arg::with_name("print-only")
-                        .long("print-only")
-                        .help("Print a table of lints to STDOUT. This does not include deprecated and internal lints. (Does not modify any files)")
+                .about("Updates lint registration and information from the source code")
+                .long_about(
+                    "Makes sure that:\n \
+                     * the lint count in README.md is correct\n \
+                     * the changelog contains markdown link references at the bottom\n \
+                     * all lint groups include the correct lints\n \
+                     * lint modules in `clippy_lints/*` are visible in `src/lib.rs` via `pub mod`\n \
+                     * all lints are registered in the lint store",
                 )
+                .arg(Arg::with_name("print-only").long("print-only").help(
+                    "Print a table of lints to STDOUT. \
+                     This does not include deprecated and internal lints. \
+                     (Does not modify any files)",
+                ))
                 .arg(
                     Arg::with_name("check")
                         .long("check")
                         .help("Checks that util/dev update_lints has been run. Used on CI."),
-                )
-            )
-            .get_matches();
+                ),
+        )
+        .get_matches();
 
     if let Some(matches) = matches.subcommand_matches("update_lints") {
         if matches.is_present("print-only") {
@@ -62,13 +56,21 @@ fn print_lints() {
     let grouped_by_lint_group = Lint::by_lint_group(&usable_lints);
 
     for (lint_group, mut lints) in grouped_by_lint_group {
-        if lint_group == "Deprecated" { continue; }
+        if lint_group == "Deprecated" {
+            continue;
+        }
         println!("\n## {}", lint_group);
 
         lints.sort_by_key(|l| l.name.clone());
 
         for lint in lints {
-            println!("* [{}]({}#{}) ({})", lint.name, clippy_dev::DOCS_LINK.clone(), lint.name, lint.desc);
+            println!(
+                "* [{}]({}#{}) ({})",
+                lint.name,
+                clippy_dev::DOCS_LINK.clone(),
+                lint.name,
+                lint.desc
+            );
         }
     }
 
@@ -99,8 +101,9 @@ fn update_lints(update_mode: &UpdateMode) {
         "<!-- end autogenerated links to lint list -->",
         false,
         update_mode == &UpdateMode::Change,
-        || { gen_changelog_lint_list(lint_list.clone()) }
-    ).changed;
+        || gen_changelog_lint_list(lint_list.clone()),
+    )
+    .changed;
 
     file_change |= replace_region_in_file(
         "../clippy_lints/src/lib.rs",
@@ -108,8 +111,9 @@ fn update_lints(update_mode: &UpdateMode) {
         "end deprecated lints",
         false,
         update_mode == &UpdateMode::Change,
-        || { gen_deprecated(&lint_list) }
-    ).changed;
+        || gen_deprecated(&lint_list),
+    )
+    .changed;
 
     file_change |= replace_region_in_file(
         "../clippy_lints/src/lib.rs",
@@ -117,8 +121,9 @@ fn update_lints(update_mode: &UpdateMode) {
         "end lints modules",
         false,
         update_mode == &UpdateMode::Change,
-        || { gen_modules_list(lint_list.clone()) }
-    ).changed;
+        || gen_modules_list(lint_list.clone()),
+    )
+    .changed;
 
     // Generate lists of lints in the clippy::all lint group
     file_change |= replace_region_in_file(
@@ -129,16 +134,18 @@ fn update_lints(update_mode: &UpdateMode) {
         update_mode == &UpdateMode::Change,
         || {
             // clippy::all should only include the following lint groups:
-            let all_group_lints = usable_lints.clone().into_iter().filter(|l| {
-                l.group == "correctness" ||
-                  l.group == "style" ||
-                  l.group == "complexity" ||
-                  l.group == "perf"
-            }).collect();
+            let all_group_lints = usable_lints
+                .clone()
+                .into_iter()
+                .filter(|l| {
+                    l.group == "correctness" || l.group == "style" || l.group == "complexity" || l.group == "perf"
+                })
+                .collect();
 
             gen_lint_group_list(all_group_lints)
-        }
-    ).changed;
+        },
+    )
+    .changed;
 
     // Generate the list of lints for all other lint groups
     for (lint_group, lints) in Lint::by_lint_group(&usable_lints) {
@@ -148,12 +155,16 @@ fn update_lints(update_mode: &UpdateMode) {
             r#"\]\);"#,
             false,
             update_mode == &UpdateMode::Change,
-            || { gen_lint_group_list(lints.clone()) }
-        ).changed;
+            || gen_lint_group_list(lints.clone()),
+        )
+        .changed;
     }
 
     if update_mode == &UpdateMode::Check && file_change {
-      println!("Not all lints defined properly. Please run `util/dev update_lints` to make sure all lints are defined properly.");
-      std::process::exit(1);
+        println!(
+            "Not all lints defined properly. \
+             Please run `util/dev update_lints` to make sure all lints are defined properly."
+        );
+        std::process::exit(1);
     }
 }
