@@ -3,7 +3,7 @@ use if_chain::if_chain;
 use rustc::hir::*;
 use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
 use rustc::ty;
-use rustc::{declare_tool_lint, lint_array};
+use rustc::{declare_lint_pass, declare_tool_lint};
 
 declare_clippy_lint! {
     /// **What it does:** Checks for creation of references to zeroed or uninitialized memory.
@@ -26,17 +26,7 @@ const UNINIT_REF_SUMMARY: &str = "reference to uninitialized memory";
 const HELP: &str = "Creation of a null reference is undefined behavior; \
                     see https://doc.rust-lang.org/reference/behavior-considered-undefined.html";
 
-pub struct InvalidRef;
-
-impl LintPass for InvalidRef {
-    fn get_lints(&self) -> LintArray {
-        lint_array!(INVALID_REF)
-    }
-
-    fn name(&self) -> &'static str {
-        "InvalidRef"
-    }
-}
+declare_lint_pass!(InvalidRef => [INVALID_REF]);
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for InvalidRef {
     fn check_expr(&mut self, cx: &LateContext<'a, 'tcx>, expr: &'tcx Expr) {
@@ -45,14 +35,14 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for InvalidRef {
             if let ExprKind::Path(ref qpath) = path.node;
             if args.len() == 0;
             if let ty::Ref(..) = cx.tables.expr_ty(expr).sty;
-            if let Some(def_id) = cx.tables.qpath_def(qpath, path.hir_id).opt_def_id();
+            if let Some(def_id) = cx.tables.qpath_res(qpath, path.hir_id).opt_def_id();
             then {
-                let msg = if match_def_path(cx.tcx, def_id, &paths::MEM_ZEROED) |
-                             match_def_path(cx.tcx, def_id, &paths::INIT)
+                let msg = if match_def_path(cx, def_id, &paths::MEM_ZEROED) |
+                             match_def_path(cx, def_id, &paths::INIT)
                 {
                     ZERO_REF_SUMMARY
-                } else if match_def_path(cx.tcx, def_id, &paths::MEM_UNINIT) |
-                          match_def_path(cx.tcx, def_id, &paths::UNINIT)
+                } else if match_def_path(cx, def_id, &paths::MEM_UNINIT) |
+                          match_def_path(cx, def_id, &paths::UNINIT)
                 {
                     UNINIT_REF_SUMMARY
                 } else {
