@@ -1,23 +1,12 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
 #![feature(rustc_private)]
 
-extern crate clippy_lints;
 extern crate syntax;
 use std::collections::Bound;
 
 #[test]
 fn test_overlapping() {
     use clippy_lints::matches::overlapping;
-    use crate::syntax::source_map::DUMMY_SP;
+    use syntax::source_map::DUMMY_SP;
 
     let sp = |s, e| clippy_lints::matches::SpannedRange {
         span: DUMMY_SP,
