@@ -37,27 +37,9 @@ fn main() {
     let ok16 = 0xFE_BAFE_ABAB_ABCD;
     let ok17 = 0x123_4567_8901_usize;
 
-    let fail9 = 0xabcdef;
-    let fail10 = 0xBAFEBAFE;
-    let fail11 = 0xabcdeff;
-    let fail12 = 0xabcabcabcabcabcabc;
     let fail13 = 0x1_23456_78901_usize;
 
-    let fail14 = 2_32;
-    let fail15 = 4_64;
-    let fail16 = 7_8;
-    let fail17 = 23_16;
-    let ok18 = 23_128;
     let fail19 = 12_3456_21;
-    let fail20 = 2__8;
-    let fail21 = 4___16;
     let fail22 = 3__4___23;
     let fail23 = 3__16___23;
-
-    let fail24 = 12.34_64;
-    let fail25 = 1E2_32;
-    let fail26 = 43E7_64;
-    let fail27 = 243E17_32;
-    let fail28 = 241251235E723_64;
-    let fail29 = 42279.911_32;
 }
