@@ -7,39 +7,39 @@ pub trait EvalContextExt<'tcx> {
     fn ptr_op(
         &self,
         bin_op: mir::BinOp,
-        left: ImmTy<'tcx, Borrow>,
-        right: ImmTy<'tcx, Borrow>,
-    ) -> EvalResult<'tcx, (Scalar<Borrow>, bool)>;
+        left: ImmTy<'tcx, Tag>,
+        right: ImmTy<'tcx, Tag>,
+    ) -> EvalResult<'tcx, (Scalar<Tag>, bool)>;
 
     fn ptr_int_arithmetic(
         &self,
         bin_op: mir::BinOp,
-        left: Pointer<Borrow>,
+        left: Pointer<Tag>,
         right: u128,
         signed: bool,
-    ) -> EvalResult<'tcx, (Scalar<Borrow>, bool)>;
+    ) -> EvalResult<'tcx, (Scalar<Tag>, bool)>;
 
     fn ptr_eq(
         &self,
-        left: Scalar<Borrow>,
-        right: Scalar<Borrow>,
+        left: Scalar<Tag>,
+        right: Scalar<Tag>,
     ) -> EvalResult<'tcx, bool>;
 
     fn pointer_offset_inbounds(
         &self,
-        ptr: Scalar<Borrow>,
+        ptr: Scalar<Tag>,
         pointee_ty: Ty<'tcx>,
         offset: i64,
-    ) -> EvalResult<'tcx, Scalar<Borrow>>;
+    ) -> EvalResult<'tcx, Scalar<Tag>>;
 }
 
 impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for super::MiriEvalContext<'a, 'mir, 'tcx> {
     fn ptr_op(
         &self,
         bin_op: mir::BinOp,
-        left: ImmTy<'tcx, Borrow>,
-        right: ImmTy<'tcx, Borrow>,
-    ) -> EvalResult<'tcx, (Scalar<Borrow>, bool)> {
+        left: ImmTy<'tcx, Tag>,
+        right: ImmTy<'tcx, Tag>,
+    ) -> EvalResult<'tcx, (Scalar<Tag>, bool)> {
         use rustc::mir::BinOp::*;
 
         trace!("ptr_op: {:?} {:?} {:?}", *left, bin_op, *right);
@@ -136,8 +136,8 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for super::MiriEvalContext<'a, 'mir, '
 
     fn ptr_eq(
         &self,
-        left: Scalar<Borrow>,
-        right: Scalar<Borrow>,
+        left: Scalar<Tag>,
+        right: Scalar<Tag>,
     ) -> EvalResult<'tcx, bool> {
         let size = self.pointer_size();
         Ok(match (left, right) {
@@ -152,8 +152,9 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for super::MiriEvalContext<'a, 'mir, '
                     // This accepts one-past-the end. Thus, there is still technically
                     // some non-determinism that we do not fully rule out when two
                     // allocations sit right next to each other. The C/C++ standards are
-                    // somewhat fuzzy about this case, so I think for now this check is
-                    // "good enough".
+                    // somewhat fuzzy about this case, so pragmatically speaking I think
+                    // for now this check is "good enough".
+                    // FIXME: Once we support intptrcast, we could try to fix these holes.
                     // Dead allocations in miri cannot overlap with live allocations, but
                     // on read hardware this can easily happen. Thus for comparisons we require
                     // both pointers to be live.
@@ -169,8 +170,17 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for super::MiriEvalContext<'a, 'mir, '
                 assert_eq!(size as u64, self.pointer_size().bytes());
                 let bits = bits as u64;
 
-                // Case I: Comparing with NULL.
-                if bits == 0 {
+                // Case I: Comparing real pointers with "small" integers.
+                // Really we should only do this for NULL, but pragmatically speaking on non-bare-metal systems,
+                // an allocation will never be at the very bottom of the address space.
+                // Such comparisons can arise when comparing empty slices, which sometimes are "fake"
+                // integer pointers (okay because the slice is empty) and sometimes point into a
+                // real allocation.
+                // The most common source of such integer pointers is `NonNull::dangling()`, which
+                // equals the type's alignment. i128 might have an alignment of 16 bytes, but few types have
+                // alignment 32 or higher, hence the limit of 32.
+                // FIXME: Once we support intptrcast, we could try to fix these holes.
+                if bits < 32 {
                     // Test if the ptr is in-bounds. Then it cannot be NULL.
                     // Even dangling pointers cannot be NULL.
                     if self.memory().check_bounds_ptr(ptr, InboundsCheck::MaybeDead).is_ok() {
@@ -223,13 +233,13 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for super::MiriEvalContext<'a, 'mir, '
     fn ptr_int_arithmetic(
         &self,
         bin_op: mir::BinOp,
-        left: Pointer<Borrow>,
+        left: Pointer<Tag>,
         right: u128,
         signed: bool,
-    ) -> EvalResult<'tcx, (Scalar<Borrow>, bool)> {
+    ) -> EvalResult<'tcx, (Scalar<Tag>, bool)> {
         use rustc::mir::BinOp::*;
 
-        fn map_to_primval((res, over): (Pointer<Borrow>, bool)) -> (Scalar<Borrow>, bool) {
+        fn map_to_primval((res, over): (Pointer<Tag>, bool)) -> (Scalar<Tag>, bool) {
             (Scalar::Ptr(res), over)
         }
 
@@ -317,10 +327,10 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for super::MiriEvalContext<'a, 'mir, '
     /// allocation, and all the remaining integers pointers their own allocation.
     fn pointer_offset_inbounds(
         &self,
-        ptr: Scalar<Borrow>,
+        ptr: Scalar<Tag>,
         pointee_ty: Ty<'tcx>,
         offset: i64,
-    ) -> EvalResult<'tcx, Scalar<Borrow>> {
+    ) -> EvalResult<'tcx, Scalar<Tag>> {
         // FIXME: assuming here that type size is less than `i64::max_value()`.
         let pointee_size = self.layout_of(pointee_ty)?.size.bytes() as i64;
         let offset = offset
