@@ -10,6 +10,6 @@ pub fn main() {
     unsafe {
         use crate::rusti::*;
 
-        ctlz_nonzero(0u8); //~ ERROR constant evaluation error: ctlz_nonzero called on 0
+        ctlz_nonzero(0u8); //~ ERROR ctlz_nonzero called on 0
     }
 }
