@@ -7,5 +7,5 @@ pub(crate) struct NonZero<T>(pub(crate) T);
 
 fn main() {
     // Make sure that we detect this even when no function call is happening along the way
-    let _x = Some(NonZero(0)); //~ ERROR encountered 0, but expected something greater or equal to 1
+    let _x = Some(unsafe { NonZero(0) }); //~ ERROR encountered 0, but expected something greater or equal to 1
 }
