@@ -8,6 +8,9 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
+use std::env;
+use std::path::Path;
+
 fn main() {
     println!("cargo:rerun-if-changed=build.rs");
     println!("cargo:rerun-if-env-changed=CFG_RELEASE_CHANNEL");
@@ -20,4 +23,8 @@ fn main() {
         "cargo:rustc-env=COMMIT_DATE={}",
         rustc_tools_util::get_commit_date().unwrap_or_default()
     );
+    println!(
+        "cargo:rustc-env=FIXTURES_DIR={}",
+        Path::new(&env::var("CARGO_MANIFEST_DIR").unwrap()).join("tests/fixtures").display()
+    );
 }
