@@ -365,12 +365,12 @@ impl InitActionContext {
         self.build_queue.block_on_build();
     }
 
-    /// Returns true if there are no builds pending or in progress.
+    /// Returns `true` if there are no builds pending or in progress.
     fn build_ready(&self) -> bool {
         self.build_queue.build_ready()
     }
 
-    /// Returns true if there are no builds or post-build (analysis) tasks pending
+    /// Returns `true` if there are no builds or post-build (analysis) tasks pending
     /// or in progress.
     fn analysis_ready(&self) -> bool {
         self.active_build_count.load(Ordering::SeqCst) == 0
@@ -512,7 +512,7 @@ impl FileWatch {
         // Find any Cargo.tomls in the project
         for entry in WalkDir::new(project_str)
             .into_iter()
-            .filter_map(|e| e.ok())
+            .filter_map(Result::ok)
             .filter(|e| e.file_name() == "Cargo.toml")
         {
             watchers.push(watcher(entry.path().display().to_string()));
