@@ -1,13 +1,3 @@
-// Copyright 2017 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 use log::{debug, trace};
 use serde;
 use serde_json;
@@ -17,12 +7,12 @@ use crate::lsp_data::{LSPNotification, LSPRequest};
 
 use std::fmt;
 use std::io::{self, BufRead, Write};
-use std::sync::atomic::{AtomicU64, Ordering};
+use std::sync::atomic::{AtomicU32, Ordering};
 use std::sync::Arc;
 
 use jsonrpc_core::{self as jsonrpc, response, version, Id};
 
-/// A trait for anything that can read language server input messages.
+/// Anything that can read language server input messages.
 pub trait MessageReader {
     /// Read the next input message.
     fn read_message(&self) -> Option<String>;
@@ -50,7 +40,7 @@ impl MessageReader for StdioMsgReader {
 // The input is expected to provide a message as described by "Base Protocol" of Language Server
 // Protocol.
 fn read_message<R: BufRead>(input: &mut R) -> Result<String, io::Error> {
-    // Read in the "Content-Length: xx" part
+    // Read in the "Content-Length: xx" part.
     let mut size: Option<usize> = None;
     loop {
         let mut buffer = String::new();
@@ -71,7 +61,7 @@ fn read_message<R: BufRead>(input: &mut R) -> Result<String, io::Error> {
 
         let res: Vec<&str> = buffer.split(' ').collect();
 
-        // Make sure header is valid
+        // Make sure header is valid.
         if res.len() != 2 {
             return Err(io::Error::new(
                 io::ErrorKind::InvalidData,
@@ -116,29 +106,28 @@ fn read_message<R: BufRead>(input: &mut R) -> Result<String, io::Error> {
     String::from_utf8(content).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
 }
 
-/// Anything that can send notifications and responses to a language server
-/// client.
+/// Anything that can send notifications and responses to a language server client.
 pub trait Output: Sync + Send + Clone + 'static {
-    /// Send a response string along the output.
+    /// Sends a response string along the output.
     fn response(&self, output: String);
 
-    /// Get a new unique ID.
+    /// Gets a new unique ID.
     fn provide_id(&self) -> RequestId;
 
-    /// Notify the client of a failure.
+    /// Notifies the client of a failure.
     fn failure(&self, id: jsonrpc::Id, error: jsonrpc::Error) {
         let response = response::Failure { jsonrpc: Some(version::Version::V2), id, error };
 
         self.response(serde_json::to_string(&response).unwrap());
     }
 
-    /// Notify the client of a failure with the given diagnostic message.
+    /// Notifies the client of a failure with the given diagnostic message.
     fn failure_message<M: Into<String>>(&self, id: RequestId, code: jsonrpc::ErrorCode, msg: M) {
         let error = jsonrpc::Error { code, message: msg.into(), data: None };
         self.failure(Id::from(&id), error);
     }
 
-    /// Send a successful response or notification along the output.
+    /// Sends a successful response or notification along the output.
     fn success<D: ::serde::Serialize + fmt::Debug>(&self, id: RequestId, data: &D) {
         let data = match serde_json::to_string(data) {
             Ok(data) => data,
@@ -159,7 +148,7 @@ pub trait Output: Sync + Send + Clone + 'static {
         self.response(output);
     }
 
-    /// Send a notification along the output.
+    /// Sends a notification along the output.
     fn notify<A>(&self, notification: Notification<A>)
     where
         A: LSPNotification,
@@ -182,13 +171,13 @@ pub trait Output: Sync + Send + Clone + 'static {
 /// An output that sends notifications and responses on `stdout`.
 #[derive(Clone)]
 pub(super) struct StdioOutput {
-    next_id: Arc<AtomicU64>,
+    next_id: Arc<AtomicU32>,
 }
 
 impl StdioOutput {
-    /// Construct a new `stdout` output.
+    /// Constructs a new `stdout` output.
     pub(crate) fn new() -> StdioOutput {
-        StdioOutput { next_id: Arc::new(AtomicU64::new(1)) }
+        StdioOutput { next_id: Arc::new(AtomicU32::new(1).into()) }
     }
 }
 
@@ -205,7 +194,7 @@ impl Output for StdioOutput {
     }
 
     fn provide_id(&self) -> RequestId {
-        RequestId::Num(self.next_id.fetch_add(1, Ordering::SeqCst))
+        RequestId::Num(self.next_id.fetch_add(1, Ordering::SeqCst).into())
     }
 }
 
