@@ -20,6 +20,7 @@ fn initialize_params(root_path: &Path) -> InitializeParams {
         initialization_options: None,
         capabilities: ClientCapabilities {
             workspace: None,
+            window: Some(WindowClientCapabilities { progress: Some(true) }),
             text_document: None,
             experimental: None,
         },
@@ -503,6 +504,7 @@ fn client_completion_suggests_arguments_in_statements() {
             initialization_options: None,
             capabilities: lsp_types::ClientCapabilities {
                 workspace: None,
+                window: Some(WindowClientCapabilities { progress: Some(true) }),
                 text_document: Some(TextDocumentClientCapabilities {
                     completion: Some(CompletionCapability {
                         completion_item: Some(CompletionItemCapability {
@@ -1763,9 +1765,9 @@ fn client_reformat() {
     assert_eq!(result.unwrap()[0], TextEdit {
         range: Range {
             start: Position { line: 0, character: 0 },
-            end: Position { line: 1, character: u64::max_value() },
+            end: Position { line: 2, character: 0 },
         },
-        new_text: "pub mod foo;\npub fn main() {\n    let world = \"world\";\n    println!(\"Hello, {}!\", world);\n}".to_string(),
+        new_text: "pub mod foo;\npub fn main() {\n    let world = \"world\";\n    println!(\"Hello, {}!\", world);\n}\n".to_string(),
     });
 }
 
@@ -1802,14 +1804,15 @@ fn client_reformat_with_range() {
     let newline = if cfg!(windows) { "\r\n" } else { "\n" };
     let formatted = r#"pub fn main() {
     let world1 = "world";
-    println!("Hello, {}!", world1);"#
+    println!("Hello, {}!", world1);
+"#
     .replace("\r", "")
     .replace("\n", newline);
 
     let edits = result.unwrap();
     assert_eq!(edits.len(), 2);
     assert_eq!(edits[0].new_text, formatted);
-    assert_eq!(edits[1].new_text, "");
+    assert_eq!(edits[1].new_text, newline);
 }
 
 #[test]
