@@ -1,37 +1,27 @@
-// Copyright 2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 //! Reorder items.
 //!
-//! `mod`, `extern crate` and `use` declarations are reorderd in alphabetical
+//! `mod`, `extern crate` and `use` declarations are reordered in alphabetical
 //! order. Trait items are reordered in pre-determined order (associated types
 //! and constants comes before methods).
 
 // FIXME(#2455): Reorder trait items.
 
-use config::Config;
+use std::cmp::{Ord, Ordering};
+
 use syntax::{ast, attr, source_map::Span};
 
-use attr::filter_inline_attrs;
-use comment::combine_strs_with_missing_comments;
-use imports::{merge_use_trees, UseTree};
-use items::{is_mod_decl, rewrite_extern_crate, rewrite_mod};
-use lists::{itemize_list, write_list, ListFormatting, ListItem};
-use rewrite::{Rewrite, RewriteContext};
-use shape::Shape;
-use source_map::LineRangeUtils;
-use spanned::Spanned;
-use utils::mk_sp;
-use visitor::FmtVisitor;
-
-use std::cmp::{Ord, Ordering};
+use crate::attr::filter_inline_attrs;
+use crate::comment::combine_strs_with_missing_comments;
+use crate::config::Config;
+use crate::imports::{merge_use_trees, UseTree};
+use crate::items::{is_mod_decl, rewrite_extern_crate, rewrite_mod};
+use crate::lists::{itemize_list, write_list, ListFormatting, ListItem};
+use crate::rewrite::{Rewrite, RewriteContext};
+use crate::shape::Shape;
+use crate::source_map::LineRangeUtils;
+use crate::spanned::Spanned;
+use crate::utils::{contains_skip, mk_sp};
+use crate::visitor::FmtVisitor;
 
 /// Choose the ordering between the given two items.
 fn compare_items(a: &ast::Item, b: &ast::Item) -> Ordering {
@@ -42,8 +32,10 @@ fn compare_items(a: &ast::Item, b: &ast::Item) -> Ordering {
         (&ast::ItemKind::ExternCrate(ref a_name), &ast::ItemKind::ExternCrate(ref b_name)) => {
             // `extern crate foo as bar;`
             //               ^^^ Comparing this.
-            let a_orig_name = a_name.map_or_else(|| a.ident.as_str(), |symbol| symbol.as_str());
-            let b_orig_name = b_name.map_or_else(|| b.ident.as_str(), |symbol| symbol.as_str());
+            let a_orig_name =
+                a_name.map_or_else(|| a.ident.as_str(), syntax_pos::symbol::Symbol::as_str);
+            let b_orig_name =
+                b_name.map_or_else(|| b.ident.as_str(), syntax_pos::symbol::Symbol::as_str);
             let result = a_orig_name.cmp(&b_orig_name);
             if result != Ordering::Equal {
                 return result;
@@ -63,7 +55,7 @@ fn compare_items(a: &ast::Item, b: &ast::Item) -> Ordering {
 }
 
 fn wrap_reorderable_items(
-    context: &RewriteContext,
+    context: &RewriteContext<'_>,
     list_items: &[ListItem],
     shape: Shape,
 ) -> Option<String> {
@@ -74,7 +66,7 @@ fn wrap_reorderable_items(
 }
 
 fn rewrite_reorderable_item(
-    context: &RewriteContext,
+    context: &RewriteContext<'_>,
     item: &ast::Item,
     shape: Shape,
 ) -> Option<String> {
@@ -93,13 +85,28 @@ fn rewrite_reorderable_item(
         _ => return None,
     };
 
-    combine_strs_with_missing_comments(context, &attrs_str, &item_str, missed_span, shape, false)
+    let allow_extend = if attrs.len() == 1 {
+        let line_len = attrs_str.len() + 1 + item_str.len();
+        !attrs.first().unwrap().is_sugared_doc
+            && context.config.inline_attribute_width() >= line_len
+    } else {
+        false
+    };
+
+    combine_strs_with_missing_comments(
+        context,
+        &attrs_str,
+        &item_str,
+        missed_span,
+        shape,
+        allow_extend,
+    )
 }
 
 /// Rewrite a list of items with reordering. Every item in `items` must have
 /// the same `ast::ItemKind`.
 fn rewrite_reorderable_items(
-    context: &RewriteContext,
+    context: &RewriteContext<'_>,
     reorderable_items: &[&ast::Item],
     shape: Shape,
     span: Span,
@@ -186,7 +193,9 @@ enum ReorderableItemKind {
 impl ReorderableItemKind {
     fn from(item: &ast::Item) -> Self {
         match item.node {
-            _ if contains_macro_use_attr(item) => ReorderableItemKind::Other,
+            _ if contains_macro_use_attr(item) | contains_skip(&item.attrs) => {
+                ReorderableItemKind::Other
+            }
             ast::ItemKind::ExternCrate(..) => ReorderableItemKind::ExternCrate,
             ast::ItemKind::Mod(..) if is_mod_decl(item) => ReorderableItemKind::Mod,
             ast::ItemKind::Use(..) => ReorderableItemKind::Use,
@@ -261,7 +270,7 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
         item_length
     }
 
-    /// Visit and format the given items. Items are reordered If they are
+    /// Visits and format the given items. Items are reordered If they are
     /// consecutive and reorderable.
     pub fn visit_items_with_reordering(&mut self, mut items: &[&ast::Item]) {
         while !items.is_empty() {
