@@ -1,5 +1,4 @@
 // rustfmt-wrap_comments: true
-// rustfmt-error_on_line_overflow: false
 // Enums test
 
 #[atrr]
@@ -49,7 +48,7 @@ pub enum EnumWithAttributes {
     //This is a pre comment AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
     TupleVar(usize, usize, usize), // AAAA AAAAAAAAAAAAAAAAAAAAA AAAAAAAAAAAAAAAAAAAAAAAA AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
     // Pre Comment
-    #[rustfmt_skip]
+    #[rustfmt::skip]
     SkippedItem(String,String,), // Post-comment
     #[another_attr]
     #[attr2]
@@ -190,7 +189,7 @@ pub enum QlError {
     // (from, to)
     #[fail(display = "Translation error: from {} to {}", 0, 1)] TranslationError(String, String),
     // (kind, input, expected)
-    #[fail(display = "Could not find {}: Found: {}, expected: {:?}", 0, 1, 2)] ResolveError(&'static str, String, Option<String>),
+    #[fail(display = "aaaaaaaaaaaaCould not find {}: Found: {}, expected: {:?}", 0, 1, 2)] ResolveError(&'static str, String, Option<String>),
 }
 
 // #2594
